#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <strings.h>
#include <string.h>
#include "flowtee.h"

/* in newflow versions 1 and 5 the packets are < 1500 bytes long */
#define PLEN 1500
#define MAXRECIPS 50
#define DEBUG 0

int main(int argc, char **argv) {
  int sockfd, len, ret, numrecips, i, localport;
  struct sockaddr_in servaddr, cliaddr;
  char pbuff[PLEN], *ptr;
  flowtee_recip r[MAXRECIPS];

  if (argc<3 || (argc>1 && !strcmp(argv[1], "-h"))) {
    usage();
    exit(0);
  }

  /* get the local port */
  if (strchr(argv[1], ':')) {
    usage();
    exit(1);
  }
  localport=atoi(argv[1]);
  argc-=2;
  argv+=2;

  /* parse the argvs into recipients */
  numrecips=0;
  for (i=0; i<argc; i++) {
    ptr=strchr(argv[i], ':');
    if (!ptr) {
      usage();
      exit(-1);
    }
    ptr[0]='\0';
    ptr++;
    flowtee_recip_create(&r[i], argv[i], atoi(ptr));
    numrecips++;
    if (DEBUG) printf("DEBUG: registered %s:%i\n", argv[i], atoi(ptr));
  }

  if (numrecips==0) {
    printf("You have not specified any recipients.\n");
    usage();
    exit(0);
  } else if (numrecips > MAXRECIPS) {
    printf("Currently the maximum number of recipients available is %i and you\n", MAXRECIPS);
    printf("have requested %i.  Please recompile if you need to use this many\n", numrecips);
    exit(1);
  }

  /* setup the listening server */
  sockfd=socket(AF_INET, SOCK_DGRAM, 0);
  memset(&servaddr, 0, sizeof(servaddr));
  servaddr.sin_family=AF_INET;
  servaddr.sin_addr.s_addr=htonl(INADDR_ANY);
  servaddr.sin_port=htons(localport);
  ret=bind(sockfd, (struct sockaddr *) &servaddr, sizeof(servaddr));
  if (ret==-1) {
    perror("binding port");
    exit(1);
  }

  /* make me a proper daemon */
  if (!DEBUG) {
    /* we should close the filedescriptors we're not using */
    
    i=fork();
    if (i == -1) {
      perror("forking");
      exit(1);
    } else if (i != 0) {
      /* parent exits */
      exit(0);
    }

    setsid();
    chdir("/");
    umask(0);
  }

  memset(&cliaddr, 0, sizeof(cliaddr));

  while (1) {
    len=sizeof(cliaddr);
    ret=recvfrom(sockfd, pbuff, PLEN, 0, (struct sockaddr *) &cliaddr, &len);
    if (ret==-1) {
      /* error */
      continue;
    }
    for (i=0; i<numrecips; i++) {
      flowtee_recip_send_data(&r[i], pbuff, ret, (struct sockaddr_in *) &cliaddr);
    }
  }

}

void usage() {
  printf("Usage: flowtee <localport> <ip:port> [<ip:port> ...]\n");
}




