#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include "flowtee.h"

int flowtee_recip_create(flowtee_recip *r, char *ip, int port) {
  int on=1;
  int ret;
  
  r->sock=socket(AF_INET, SOCK_RAW, IPPROTO_RAW);
  memset(&(r->sain), 0, sizeof(struct sockaddr_in));
  r->sain.sin_family=AF_INET;
  if ((r->sain.sin_addr.s_addr=inet_addr(ip)) == -1) {
    return(-1);
  }
  r->sain.sin_port=htons(port);

  /* turn HDRINCL on */
  on=1;
  ret=setsockopt(r->sock, IPPROTO_IP, IP_HDRINCL, &on, sizeof(on));
  if (ret<0) {
    perror("setting socket option");
    exit(1);
  }

  /* construct a default header */
  memcpy(&(r->packet),
	 "\x45\x00\x00\x20" /* vers, hlen, total len */
	 "\x00\x00\x40\x00" /* id, flags (DF), fragoff */
	 "\x0f\x11\x00\x00" /* TTL, proto */ /* TTL low on purpose */
	 "\x00\x00\x00\x00" /* src ip */
	 "\x00\x00\x00\x00" /* dest ip */
	 "\x04\x57\x00\x00" /* src port, dest port */
	 "\x00\x00\x00\x00" /* UDP length, checksum */
	 , 28);

  /* set the destination IP address */
  r->packet[16]=(unsigned char) (r->sain.sin_addr.s_addr >> 24);
  r->packet[17]=(unsigned char) ((r->sain.sin_addr.s_addr >> 16) & 255);
  r->packet[18]=(unsigned char) ((r->sain.sin_addr.s_addr >> 8) & 255);
  r->packet[19]=(unsigned char) (r->sain.sin_addr.s_addr & 255);

  /* set the destination port */
  r->packet[22]=(unsigned char) (port>>8)&255;
  r->packet[23]=(unsigned char) port&255;
  
  return(0);
}

struct sockaddr *flowtee_recip_get_addr(flowtee_recip *r) {
  return((struct sockaddr *) &(r->sain));
}

int flowtee_recip_send_data(flowtee_recip *r, char *buff, int len, struct sockaddr_in *from) {
  int i;
  
  /* set the src ip addr */
  r->packet[12]=(unsigned char) (from->sin_addr.s_addr >> 24);
  r->packet[13]=(unsigned char) ((from->sin_addr.s_addr >> 16) & 255);
  r->packet[14]=(unsigned char) ((from->sin_addr.s_addr >> 8) & 255);
  r->packet[15]=(unsigned char) (from->sin_addr.s_addr & 255);

  /* set the total IP len = IP header + UDP header + datalen */
  i=28+len;
  r->packet[2]=(unsigned char) (i>>8);
  r->packet[3]=(unsigned char) i&255;

  /* set the UDP packet len */
  i=8+len;
  r->packet[24]=(unsigned char) (i>>8);
  r->packet[25]=(unsigned char) i&255;

  /* write the data into the packet */
  memcpy(r->packet+28, buff, len);
  
  /* send it */
  return(sendto(r->sock, &(r->packet), len+28, 0, (struct sockaddr *) &(r->sain), sizeof(struct sockaddr)));
}
